@@ -140,7 +140,7 @@ static const char* pystr_to_ascii_c_str(const py::str& string) {
 }
 
 static py::int_ object_to_py_long(const py::object& value) {
-  PyObject* result_ptr = PyNumber_Index(value.ptr());
+  PyObject* result_ptr = PyNumber_Long(value.ptr());
   if (!result_ptr) {
     throw py::error_already_set();
   }
@@ -161,39 +161,53 @@ class Int : public BaseInt {
   Int(const py::str& value, std::size_t base)
       : BaseInt(pystr_to_ascii_c_str(value), base) {}
 
+  operator bool() const noexcept { return BaseInt::operator bool(); }
+
   operator py::int_() const {
     return py::reinterpret_steal<py::int_>((PyObject*)as_PyLong());
   }
 
-  Int operator%(const Int& divisor) const {
-    return Int(BaseInt::operator%(divisor));
-  }
+  const Int& operator+() const noexcept { return *this; }
 
-  const Int& operator+() const { return *this; }
-
-  Int operator+(const Int& other) const {
+  Int operator+(const Int& other) const noexcept {
     return Int(BaseInt::operator+(other));
   }
 
-  Int operator~() const { return Int(BaseInt::operator~()); }
+  Int operator&(const Int& other) const noexcept {
+    return Int(BaseInt::operator&(other));
+  }
 
-  Int operator*(const Int& other) const {
+  Int operator~() const noexcept { return Int(BaseInt::operator~()); }
+
+  Int operator*(const Int& other) const noexcept {
     return Int(BaseInt::operator*(other));
   }
 
-  Int operator-() const { return Int(BaseInt::operator-()); }
+  Int operator-() const noexcept { return Int(BaseInt::operator-()); }
 
-  Int operator-(const Int& other) const {
+  Int operator-(const Int& other) const noexcept {
     return Int(BaseInt::operator-(other));
   }
 
-  Int operator/(const Int& divisor) const {
-    return Int(BaseInt::operator/(divisor));
+  Int operator|(const Int& other) const noexcept {
+    return Int(BaseInt::operator|(other));
   }
 
-  Int abs() const { return Int(BaseInt::abs()); }
+  Int operator^(const Int& other) const noexcept {
+    return Int(BaseInt::operator^(other));
+  }
 
-  PyLongObject* as_PyLong() const {
+  Int operator<<(const Int& other) const {
+    return Int(BaseInt::operator<<(other));
+  }
+
+  Int operator>>(const Int& other) const {
+    return Int(BaseInt::operator>>(other));
+  }
+
+  Int abs() const noexcept { return Int(BaseInt::abs()); }
+
+  PyLongObject* as_PyLong() const noexcept {
     const std::vector<BaseInt::Digit>& digits = this->digits();
     std::vector<digit> result_digits =
         cppbuiltins::binary_digits_to_binary_base<
@@ -209,9 +223,15 @@ class Int : public BaseInt {
     return result;
   }
 
+  Int bit_length() const noexcept { return Int(BaseInt::bit_length()); }
+
+  Int floor_divide(const Int& divisor) const {
+    return Int(BaseInt::floor_divide(divisor));
+  }
+
   Int gcd(const Int& other) const { return Int(BaseInt::gcd(other)); }
 
-  Py_hash_t hash() const {
+  Py_hash_t hash() const noexcept {
     const std::vector<BaseInt::Digit>& digits = this->digits();
     if (digits.size() == 1) {
       if (is_positive())
@@ -236,11 +256,15 @@ class Int : public BaseInt {
 
   Int invmod(const Int& divisor) const { return Int(BaseInt::invmod(divisor)); }
 
+  Int mod(const Int& divisor) const { return Int(BaseInt::mod(divisor)); }
+
   Int power(const Int& exponent) const { return Int(BaseInt::power(exponent)); }
 
   Int power_modulo(const Int& exponent, const Int& modulus) const {
     return Int(BaseInt::power_modulo(exponent, modulus));
   }
+
+  std::string repr() const noexcept { return BaseInt::repr<10>(); }
 };
 
 static std::ostream& operator<<(std::ostream& stream, const Int& value) {
@@ -248,33 +272,24 @@ static std::ostream& operator<<(std::ostream& stream, const Int& value) {
 }
 
 namespace cppbuiltins {
-template <>
-class Gcd<Int> {
- public:
-  Int operator()(ConstParameterFrom<Int> first,
-                 ConstParameterFrom<Int> second) const {
-    return first.gcd(second);
-  }
-};
-
 template <>
 double divide_as_double<Int>(ConstParameterFrom<Int> dividend,
                              ConstParameterFrom<Int> divisor) {
-  return dividend.divide_as_double(divisor);
+  return dividend.divide_approximately(divisor);
 }
 
 template <>
-bool is_negative<Int>(ConstParameterFrom<Int> value) {
+bool is_negative<Int>(ConstParameterFrom<Int> value) noexcept {
   return value.is_negative();
 }
 
 template <>
-bool is_one<Int>(ConstParameterFrom<Int> value) {
+bool is_one<Int>(ConstParameterFrom<Int> value) noexcept {
   return value.is_one();
 }
 
 template <>
-bool is_positive<Int>(ConstParameterFrom<Int> value) {
+bool is_positive<Int>(ConstParameterFrom<Int> value) noexcept {
   return value.is_positive();
 }
 
@@ -312,37 +327,6 @@ Py_hash_t hash_fraction(const Fraction& value) {
   return result - (result == -1);
 }
 
-static Fraction operator+(const Int& self, const Fraction& other) {
-  return Fraction(self * other.denominator() + other.numerator(),
-                  other.denominator());
-}
-
-static Fraction operator-(const Int& self, const Fraction& other) {
-  return Fraction(self * other.denominator() - other.numerator(),
-                  other.denominator());
-}
-
-static Fraction operator%(const Int& self, const Fraction& other) {
-  return Fraction((self * other.denominator()) % other.numerator(),
-                  other.denominator());
-}
-
-static Fraction operator*(const Int& self, const Fraction& other) {
-  const Int self_other_denominator_gcd = self.gcd(other.denominator());
-  return Fraction((self / self_other_denominator_gcd) * other.numerator(),
-                  other.denominator() / self_other_denominator_gcd);
-}
-
-static Fraction operator/(const Int& self, const Fraction& other) {
-  const Int self_other_numerator_gcd = self.gcd(other.numerator());
-  return Fraction((self / self_other_numerator_gcd) * other.denominator(),
-                  other.numerator() / self_other_numerator_gcd);
-}
-
-Int floor_divide(const Int& self, const Fraction& other) {
-  return (self * other.denominator()) / other.numerator();
-}
-
 static std::ostream& operator<<(std::ostream& stream, const Fraction& value) {
   return stream << C_STR(MODULE_NAME) "." FRACTION_NAME "(" << value.numerator()
                 << ", " << value.denominator() << ")";
@@ -1115,21 +1099,30 @@ PYBIND11_MODULE(MODULE_NAME, m) {
 
   static const Int ONE{1};
 
+  static const py::object Rational =
+      py::module::import("numbers").attr("Rational");
+
   py::class_<Int> PyInt(m, INT_NAME);
   PyInt.def(py::init<>())
+      .def(py::init<const Int&>(), py::arg("value"))
       .def(py::init<const py::str&, std::size_t>(), py::arg("string"),
            py::arg("base") = 10)
       .def(py::init<const py::object&>(), py::arg("value"))
+      .def("bit_length", &Int::bit_length)
       .def(py::self + py::self)
+      .def(py::self & py::self)
       .def(py::self == py::self)
       .def(~py::self)
       .def(py::self <= py::self)
       .def(py::self < py::self)
-      .def(py::self % py::self)
       .def(py::self * py::self)
       .def(-py::self)
       .def(+py::self)
       .def(py::self - py::self)
+      .def(py::self | py::self)
+      .def(py::self ^ py::self)
+      .def(py::self << py::self)
+      .def(py::self >> py::self)
       .def(py::pickle([](const Int& self) { return py::int_(self); },
                       [](const py::int_& state) { return Int(state); }))
       .def("__abs__", &Int::abs)
@@ -1148,9 +1141,10 @@ PYBIND11_MODULE(MODULE_NAME, m) {
            [](const Int& self, const py::dict&) -> Int { return self; })
       .def("__float__", &Int::operator double)
       .def("__floor__", &identity<const Int&>)
-      .def("__floordiv__", &Int::operator/, py::is_operator{})
+      .def("__floordiv__", &Int::floor_divide, py::is_operator{})
       .def("__hash__", &Int::hash)
       .def("__int__", &Int::operator py::int_)
+      .def("__mod__", &Int::mod, py::is_operator{})
       .def(
           "__pow__",
           [](const Int& base, const Int& exponent, const Int* maybe_modulus) {
@@ -1167,7 +1161,7 @@ PYBIND11_MODULE(MODULE_NAME, m) {
           },
           py::arg("exponent"), py::arg("modulus") = nullptr, py::is_operator{})
       .def("__repr__", &to_repr<Int>)
-      .def("__str__", &Int::repr<10>)
+      .def("__str__", &Int::repr)
       .def("__truediv__", &cppbuiltins::divide_as_double<Int>,
            py::is_operator{})
       .def("__trunc__", &identity<const Int&>)
@@ -1175,10 +1169,26 @@ PYBIND11_MODULE(MODULE_NAME, m) {
       .def_property_readonly("denominator",
                              [](const Int&) -> const Int& { return ONE; });
 
+  Rational.attr("register")(PyInt);
+
   py::class_<Fraction> PyFraction(m, FRACTION_NAME);
   PyFraction.def(py::init<>())
       .def(py::init<const Int&, const Int&>(), py::arg("numerator"),
            py::arg("denominator") = ONE)
+      .def(py::init<const Fraction&>(), py::arg("value"))
+      .def(
+          py::init([&](const py::object& value) {
+            if (py::isinstance(value, Rational)) {
+              return Fraction(Int(value.attr("numerator").cast<py::int_>()),
+                              Int(value.attr("denominator").cast<py::int_>()));
+            } else {
+              throw py::type_error(
+                  "Value should be a `numbers.Rational` instance but found `" +
+                  py::type::of(value).attr("__qualname__").cast<std::string>() +
+                  "`.");
+            }
+          }),
+          py::arg("value"))
       .def(py::self + py::self)
       .def(py::self + Int{})
       .def(py::self == py::self)
@@ -1189,14 +1199,11 @@ PYBIND11_MODULE(MODULE_NAME, m) {
       .def(py::self < py::self)
       .def(py::self < Int{})
       .def(Int{} < py::self)
-      .def(py::self % py::self)
-      .def(py::self % Int{})
       .def(py::self * py::self)
       .def(py::self * Int{})
       .def(-py::self)
       .def(+py::self)
       .def(Int{} + py::self)
-      .def(Int{} % py::self)
       .def(Int{} * py::self)
       .def(Int{} - py::self)
       .def(Int{} / py::self)
@@ -1234,6 +1241,11 @@ PYBIND11_MODULE(MODULE_NAME, m) {
            py::is_operator{})
       .def("__hash__", &hash_fraction)
       .def("__int__", [](const Fraction& self) { return py::int_(Int(self)); })
+      .def("__mod__",
+           py::overload_cast<const Fraction&>(&Fraction::mod, py::const_),
+           py::is_operator{})
+      .def("__mod__", py::overload_cast<const Int&>(&Fraction::mod, py::const_),
+           py::is_operator{})
       .def(
           "__pow__",
           [](const Fraction& base, const Int& exponent) {
@@ -1253,7 +1265,13 @@ PYBIND11_MODULE(MODULE_NAME, m) {
       .def(
           "__rfloordiv__",
           [](const Fraction& divisor, const Int& dividend) {
-            return floor_divide(dividend, divisor);
+            return cppbuiltins::floor_divide(dividend, divisor);
+          },
+          py::is_operator{})
+      .def(
+          "__rmod__",
+          [](const Fraction& divisor, const Int& dividend) {
+            return cppbuiltins::mod(dividend, divisor);
           },
           py::is_operator{})
       .def(
@@ -1273,6 +1291,8 @@ PYBIND11_MODULE(MODULE_NAME, m) {
       .def_property_readonly("denominator", &Fraction::denominator)
       .def_property_readonly("numerator", &Fraction::numerator);
 
+  Rational.attr("register")(PyFraction);
+
   py::class_<List> PyList(m, LIST_NAME);
   PyList.def(py::init<py::iterable>(), py::arg("values"))
       .def(py::self == py::self)
